package auth.firebase_auth_ui_login

import android.app.Activity
import androidx.annotation.NonNull
import androidx.lifecycle.Lifecycle
import androidx.lifecycle.LifecycleEventObserver
import com.firebase.ui.auth.AuthUI
import com.google.firebase.auth.FirebaseAuth
import io.flutter.embedding.engine.plugins.FlutterPlugin
import io.flutter.embedding.engine.plugins.activity.ActivityAware
import io.flutter.embedding.engine.plugins.activity.ActivityPluginBinding
import io.flutter.embedding.engine.plugins.lifecycle.FlutterLifecycleAdapter
import io.flutter.plugin.common.MethodCall
import io.flutter.plugin.common.MethodChannel
import io.flutter.plugin.common.MethodChannel.MethodCallHandler
import io.flutter.plugin.common.MethodChannel.Result
import io.flutter.plugin.common.PluginRegistry.ActivityResultListener

/** FirebaseAuthUiLoginPlugin */
class FirebaseAuthUiLoginPlugin: FlutterPlugin, MethodCallHandler, ActivityAware {
  private var channel : MethodChannel? = null
  private val requestCode = 123
  private var activity: Activity? = null
  private var result: Result? = null

  override fun onAttachedToEngine(@NonNull flutterPluginBinding: FlutterPlugin.FlutterPluginBinding) {
    channel = MethodChannel(flutterPluginBinding.binaryMessenger, "firebase_auth_ui_login")
    channel?.setMethodCallHandler(this)
  }

  override fun onMethodCall(@NonNull call: MethodCall, @NonNull result: Result) {
    if (call.method == "startMicrosoftAuthUI") {
      call.argument<String>("appName")?.let { appName ->
        val builder: AuthUI.SignInIntentBuilder = if(appName == "[DEFAULT]")
          AuthUI.getInstance().createSignInIntentBuilder().setAvailableProviders(listOf(AuthUI.IdpConfig.MicrosoftBuilder().build()))
        else
          AuthUI.getInstance(appName).createSignInIntentBuilder().setAvailableProviders(listOf(AuthUI.IdpConfig.MicrosoftBuilder().build()))

        builder.setIsSmartLockEnabled(true)
        val intent = builder.build()
        activity?.startActivityForResult(intent, requestCode)
        this.result = result
      }
    } else {
      result.notImplemented()
    }
  }

  override fun onDetachedFromEngine(@NonNull binding: FlutterPlugin.FlutterPluginBinding) {
    activity = null
    if(channel != null) {
      channel?.setMethodCallHandler(null)
      channel = null
    }
  }

  override fun onAttachedToActivity(binding: ActivityPluginBinding) {
    activity = binding.activity
    FlutterLifecycleAdapter
            .getActivityLifecycle(binding)
            .addObserver(LifecycleEventObserver { _, event ->
              when (event) {
                Lifecycle.Event.ON_CREATE -> binding.addActivityResultListener(listener)
                Lifecycle.Event.ON_DESTROY -> binding.removeActivityResultListener(listener)
                else -> {}
              }
            })
  }

  override fun onDetachedFromActivityForConfigChanges() {
    activity = null
  }

  override fun onReattachedToActivityForConfigChanges(binding: ActivityPluginBinding) {
    activity = binding.activity
  }

  override fun onDetachedFromActivity() {
    activity = null
  }

  private val listener = ActivityResultListener { requestCode, resultCode, data ->
    if (requestCode != this.requestCode) return@ActivityResultListener false
    if (result == null) return@ActivityResultListener true

    if (resultCode == Activity.RESULT_OK) {
      val user = FirebaseAuth.getInstance().currentUser
      result?.success(user != null)
    } else {
      result?.error(resultCode.toString(), "error result", null)
    }
    result = null
    true
  }
}
